@@ -45,6 +45,12 @@ _positions_to_ratios = {
     "bottom-right": (1.0, 1.0), "face": None
     }
 
+_orientation_to_rotation = {
+    3: 180,
+    6: 90,
+    8: 270
+    }
+
 _filters_to_pil = {
     "antialias": PIL.Image.ANTIALIAS,
     "bicubic": PIL.Image.BICUBIC,
@@ -99,6 +105,8 @@ class Image(object):
     def validate_degree(deg):
         if deg is None or deg == "":
             raise errors.DegreeError("Missing degree")
+        elif deg == "auto":
+            return
         elif not Image._isint(deg):
             raise errors.DegreeError("Invalid degree: %s" % deg)
         elif int(deg) < 0 or int(deg) >= 360:
@@ -137,8 +145,10 @@ class Image(object):
         elif opts["optimize"] and not Image._isint(opts["optimize"]):
             raise errors.OptimizeError(
                 "Invalid optimize: %s", str(opts["optimize"]))
-        elif not Image._isint(opts["quality"]) \
-                or int(opts["quality"]) > 100 or int(opts["quality"]) < 0:
+        elif opts["quality"] != "keep" \
+                and (not Image._isint(opts["quality"])
+                     or int(opts["quality"]) > 100
+                     or int(opts["quality"]) < 0):
             raise errors.QualityError(
                 "Invalid quality: %s", str(opts["quality"]))
         elif opts["progressive"] and not Image._isint(opts["progressive"]):
@@ -185,6 +195,18 @@ class Image(object):
         expand - Expand the output image to fit rotation
         """
         opts = Image._normalize_options(kwargs)
+
+        if deg == "auto":
+            if self._orig_format == "JPEG":
+                try:
+                    exif = self.img._getexif() or dict()
+                    deg = _orientation_to_rotation.get(exif.get(274, 0), 0)
+                except:
+                    logger.warn('unable to parse exif')
+                    deg = 0
+            else:
+                deg = 0
+
         expand = False if int(deg) % 90 == 0 else bool(int(opts["expand"]))
         self.img = self.img.rotate(360 - int(deg), expand=expand)
         return self
@@ -195,6 +217,7 @@ class Image(object):
 
         format - The format to save as: see Image.FORMATS
         optimize - The image file size should be optimized
+        progressive - The output should be progressive JPEG
         quality - The quality used to save JPEGs: integer from 1 - 100
         """
         opts = Image._normalize_options(kwargs)
@@ -203,11 +226,23 @@ class Image(object):
             fmt = opts["pil"]["format"]
         else:
             fmt = self._orig_format
-        save_kwargs = dict(quality=int(opts["quality"]))
+        save_kwargs = dict()
+
+        if Image._isint(opts["quality"]):
+            save_kwargs["quality"] = int(opts["quality"])
+
         if int(opts["optimize"]):
             save_kwargs["optimize"] = True
+
         if int(opts["progressive"]):
             save_kwargs["progressive"] = True
+
+        if self._orig_format == "JPEG":
+            self.img.format = self._orig_format
+            save_kwargs["subsampling"] = "keep"
+            if opts["quality"] == "keep":
+                save_kwargs["quality"] = "keep"
+
         try:
             self.img.save(outfile, fmt, **save_kwargs)
         except IOError as e:
@@ -366,7 +401,7 @@ def main():
     define("format", help="default format to use when saving",
            metavar="|".join(Image.FORMATS), type=str)
     define("optimize", help="default to optimize when saving", type=int)
-    define("quality", help="default jpeg quality, 0-100", type=int)
+    define("quality", help="default jpeg quality, 0-100 or keep")
     define("progressive", help="default to progressive when saving", type=int)
 
     args = parse_command_line()
